@@ -1,5 +1,3 @@
-# -*- coding: utf-8; -*-
-#
 # test/test_changesfile.py
 # Part of ‘dput’, a Debian package upload toolkit.
 #
@@ -9,16 +7,16 @@
 
 """ Unit tests for Debian upload control (‘*.changes’) files. """
 
-from __future__ import (absolute_import, unicode_literals)
-
 import collections
 import doctest
 import email.message
+import io
 import os
 import os.path
 import sys
 import tempfile
 import textwrap
+import unittest.mock
 
 import testscenarios
 import testtools
@@ -30,11 +28,9 @@ from .helper import (
         EXIT_STATUS_FAILURE,
         FakeSystemExit,
         FileDouble,
-        StringIO,
         get_file_doubles_from_fake_file_scenarios,
         make_fake_file_scenarios,
         make_unique_slug,
-        mock,
         patch_os_stat,
         patch_system_interfaces,
         setup_file_double_behaviour,
@@ -44,8 +40,6 @@ from .test_configfile import (
         set_config,
         )
 
-__metaclass__ = type
-
 
 class _FieldsMapping:
     """ A mapping to stand in for the `dict` of an `email.message.Message`. """
@@ -55,7 +49,7 @@ class _FieldsMapping:
             self._message = kwargs.pop('_message')
         except KeyError:
             raise TypeError("no ‘_message’ specified for this mapping")
-        super(_FieldsMapping, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def __len__(self, *args, **kwargs):
         return self._message.__len__(*args, **kwargs)
@@ -72,10 +66,6 @@ class _FieldsMapping:
     def __delitem__(self, *args, **kwargs):
         self._message.__delitem__(*args, **kwargs)
 
-    # Deprecated method for Python 2 compatibility.
-    def has_key(self, *args, **kwargs):
-        return self._message.has_key(*args, **kwargs)    # noqa: W601
-
     def keys(self, *args, **kwargs):
         return self._message.keys(*args, **kwargs)
 
@@ -93,7 +83,7 @@ class FakeMessage(email.message.Message, object):
     """ A fake RFC 2822 message that mocks the obsolete `rfc822.Message`. """
 
     def __init__(self, *args, **kwargs):
-        super(FakeMessage, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.dict = _FieldsMapping(_message=self)
 
 
@@ -146,14 +136,14 @@ def make_changes_file_scenarios():
     """ Make fake Debian upload control (‘*.changes’) scenarios. """
     file_path = make_changes_file_path()
 
-    fake_file_empty = StringIO()
-    fake_file_no_format = StringIO(textwrap.dedent("""\
+    fake_file_empty = io.StringIO()
+    fake_file_no_format = io.StringIO(textwrap.dedent("""\
             FOO
             BAR
             Files:
                 Lorem ipsum dolor sit amet
             """))
-    fake_file_with_signature = StringIO(textwrap.dedent("""\
+    fake_file_with_signature = io.StringIO(textwrap.dedent("""\
             -----BEGIN PGP SIGNED MESSAGE-----
             Hash: SHA1
 
@@ -171,12 +161,12 @@ def make_changes_file_scenarios():
             gibberishgibberishgibberishgibberishgibberishgibberish
             -----END PGP SIGNATURE-----
             """))
-    fake_file_with_format = StringIO(textwrap.dedent("""\
+    fake_file_with_format = io.StringIO(textwrap.dedent("""\
             Format: FOO
             Files:
                 Lorem ipsum dolor sit amet
             """))
-    fake_file_invalid = StringIO(textwrap.dedent("""\
+    fake_file_invalid = io.StringIO(textwrap.dedent("""\
             Format: FOO
             Files:
                 FOO BAR
@@ -317,10 +307,10 @@ class parse_changes_TestCase(
 
     def setUp(self):
         """ Set up test fixtures. """
-        super(parse_changes_TestCase, self).setUp()
+        super().setUp()
         patch_system_interfaces(self)
 
-        self.test_infile = StringIO()
+        self.test_infile = io.StringIO()
 
 
 class parse_changes_SuccessTestCase(parse_changes_TestCase):
@@ -389,7 +379,7 @@ class check_upload_variant_TestCase(
 
     def setUp(self):
         """ Set up test fixtures. """
-        super(check_upload_variant_TestCase, self).setUp()
+        super().setUp()
         patch_system_interfaces(self)
 
         self.set_changes_document(self.fields)
@@ -547,7 +537,7 @@ class source_check_TestCase(
 
     def setUp(self):
         """ Set up test fixtures. """
-        super(source_check_TestCase, self).setUp()
+        super().setUp()
         patch_system_interfaces(self)
 
         self.test_args = {
@@ -628,7 +618,7 @@ class verify_files_TestCase(
             host="foo",
             check_only=None,
             check_version=None,
-            unsigned_upload=None,
+            allow_unsigned_uploads=None,
             debug=None,
             )
 
@@ -664,7 +654,7 @@ class verify_files_TestCase(
 
     def setUp(self):
         """ Set up test fixtures. """
-        super(verify_files_TestCase, self).setUp()
+        super().setUp()
         patch_system_interfaces(self)
 
         self.file_double_by_path = {}
@@ -676,8 +666,10 @@ class verify_files_TestCase(
         setup_changes_file_fixtures(self)
         set_changes_file_scenario(self, 'exist-minimal')
         self.test_args.update(dict(
-                path=os.path.dirname(self.changes_file_double.path),
-                filename=os.path.basename(self.changes_file_double.path),
+                changes_file_directory=os.path.dirname(
+                    self.changes_file_double.path),
+                changes_file_name=os.path.basename(
+                    self.changes_file_double.path),
                 ))
 
         patch_os_stat(self)
@@ -702,7 +694,7 @@ class verify_files_TestCase(
 
     def patch_checksum_test(self):
         """ Patch `checksum_test` function for this test case. """
-        func_patcher = mock.patch.object(
+        func_patcher = unittest.mock.patch.object(
                 dput.dput, "checksum_test", autospec=True)
         mock_func = func_patcher.start()
         self.addCleanup(func_patcher.stop)
@@ -734,7 +726,7 @@ class verify_files_TestCase(
 
     def patch_parse_changes(self):
         """ Patch `parse_changes` function for this test case. """
-        func_patcher = mock.patch.object(
+        func_patcher = unittest.mock.patch.object(
                 dput.dput, "parse_changes", autospec=True)
         func_patcher.start()
         self.addCleanup(func_patcher.stop)
@@ -747,7 +739,7 @@ class verify_files_TestCase(
         if not hasattr(self, 'check_upload_variant_return_value'):
             self.check_upload_variant_return_value = True
 
-        func_patcher = mock.patch.object(
+        func_patcher = unittest.mock.patch.object(
                 dput.dput, "check_upload_variant", autospec=True,
                 return_value=self.check_upload_variant_return_value)
         func_patcher.start()
@@ -755,21 +747,21 @@ class verify_files_TestCase(
 
     def patch_version_check(self):
         """ Patch `version_check` function for this test case. """
-        func_patcher = mock.patch.object(
+        func_patcher = unittest.mock.patch.object(
                 dput.dput, "version_check", autospec=True)
         func_patcher.start()
         self.addCleanup(func_patcher.stop)
 
     def patch_verify_signature(self):
         """ Patch `verify_signature` function for this test case. """
-        func_patcher = mock.patch.object(
+        func_patcher = unittest.mock.patch.object(
                 dput.dput, "verify_signature", autospec=True)
         func_patcher.start()
         self.addCleanup(func_patcher.stop)
 
     def patch_source_check(self):
         """ Patch `source_check` function for this test case. """
-        func_patcher = mock.patch.object(
+        func_patcher = unittest.mock.patch.object(
                 dput.dput, "source_check", autospec=True)
         mock_func = func_patcher.start()
         self.addCleanup(func_patcher.stop)
@@ -832,7 +824,7 @@ class verify_files_TestCase(
         """ Should call `check_upload_variant` with changes document. """
         dput.dput.verify_files(**self.test_args)
         dput.dput.check_upload_variant.assert_called_with(
-                self.changes_document, mock.ANY)
+                self.changes_document, unittest.mock.ANY)
 
     def test_emits_upload_dsc_file_debug_message(self):
         """ Should emit debug message for ‘*.dsc’ file. """
@@ -915,7 +907,8 @@ class verify_files_TestCase(
                 self.expected_source_control_file_path,
                 self.runtime_config_parser,
                 self.test_args['check_only'],
-                self.test_args['unsigned_upload'], mock.ANY,
+                self.test_args['allow_unsigned_uploads'],
+                unittest.mock.ANY,
                 self.test_args['debug'])
 
     def test_calls_source_check_with_changes_document(self):
@@ -940,7 +933,7 @@ class verify_files_TestCase(
         """ Should call `checksum_test` with each upload file path. """
         dput.dput.verify_files(**self.test_args)
         expected_calls = [
-                mock.call(file_path, mock.ANY)
+                unittest.mock.call(file_path, unittest.mock.ANY)
                 for file_path in self.fake_upload_file_paths]
         dput.dput.checksum_test.assert_has_calls(
                 expected_calls, any_order=True)
@@ -1009,7 +1002,7 @@ class verify_files_TestCase(
         """ Should call `os.stat` with each upload file path. """
         dput.dput.verify_files(**self.test_args)
         expected_calls = [
-                mock.call(file_path)
+                unittest.mock.call(file_path)
                 for file_path in self.fake_upload_file_paths]
         os.stat.assert_has_calls(expected_calls, any_order=True)
 
@@ -1098,20 +1091,20 @@ class guess_upload_host_TestCase(
 
     changes_file_scenarios = [
             ('no-distribution', {
-                'fake_file': StringIO(textwrap.dedent("""\
+                'fake_file': io.StringIO(textwrap.dedent("""\
                     Files:
                         Lorem ipsum dolor sit amet
                     """)),
                 }),
             ('distribution-spam', {
-                'fake_file': StringIO(textwrap.dedent("""\
+                'fake_file': io.StringIO(textwrap.dedent("""\
                     Distribution: spam
                     Files:
                         Lorem ipsum dolor sit amet
                     """)),
                 }),
             ('distribution-beans', {
-                'fake_file': StringIO(textwrap.dedent("""\
+                'fake_file': io.StringIO(textwrap.dedent("""\
                     Distribution: beans
                     Files:
                         Lorem ipsum dolor sit amet
@@ -1163,7 +1156,7 @@ class guess_upload_host_TestCase(
 
     def setUp(self):
         """ Set up test fixtures. """
-        super(guess_upload_host_TestCase, self).setUp()
+        super().setUp()
         patch_system_interfaces(self)
 
         set_config(
@@ -1181,8 +1174,10 @@ class guess_upload_host_TestCase(
     def set_test_args(self):
         """ Set the arguments for the test call to the function. """
         self.test_args = dict(
-                path=os.path.dirname(self.changes_file_double.path),
-                filename=os.path.basename(self.changes_file_double.path),
+                changes_file_directory=os.path.dirname(
+                    self.changes_file_double.path),
+                changes_file_name=os.path.basename(
+                    self.changes_file_double.path),
                 config=self.runtime_config_parser,
                 )
 
@@ -1215,7 +1210,7 @@ class guess_upload_host_TestCase(
         result = dput.dput.guess_upload_host(**self.test_args)
         self.assertEqual(self.expected_host, result)
 
-    @mock.patch.object(dput.dput, 'debug', True)
+    @unittest.mock.patch.object(dput.dput, 'debug', True)
     def test_emits_debug_message_for_host(self):
         """ Should emit a debug message for the discovered host. """
         config_parser = self.runtime_config_parser
@@ -1231,7 +1226,7 @@ class guess_upload_host_TestCase(
         self.assertIn(expected_output, sys.stdout.getvalue())
 
 
-# Copyright © 2015–2016 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2022 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
