@@ -18,15 +18,19 @@ import tempfile
 import textwrap
 import unittest.mock
 
+import testscenarios
 import testtools
 import testtools.matchers
+import xdg
 
+import dput.configfile
 import dput.dput
 
 from .helper import (
         EXIT_STATUS_FAILURE,
         FakeSystemExit,
         FileDouble,
+        patch_os_path_exists,
         patch_system_interfaces,
         setup_file_double_behaviour,
         )
@@ -66,8 +70,11 @@ def make_config_file_scenarios():
         """
 
     runtime_config_file_path = tempfile.mktemp()
-    global_config_file_path = os.path.join(os.path.sep, "etc", "dput.cf")
-    user_config_file_path = os.path.join(os.path.expanduser("~"), ".dput.cf")
+    global_config_file_path = os.path.join(os.path.sep, 'etc', 'dput.cf')
+    user_xdg_config_file_path = os.path.join(
+            xdg.xdg_config_home(), 'dput', 'dput.cf')
+    user_home_config_file_path = os.path.join(
+            os.path.expanduser('~'), '.dput.cf')
 
     fake_file_empty = io.StringIO()
     fake_file_bogus = io.StringIO("b0gUs")
@@ -157,18 +164,28 @@ def make_config_file_scenarios():
                 'file_double_params': dict(
                     path=runtime_config_file_path,
                     fake_file=fake_file_minimal),
+                'os_path_exists_scenario_name': 'exist',
                 'open_scenario_name': 'okay',
                 },
             'global': {
                 'file_double_params': dict(
                     path=global_config_file_path,
                     fake_file=fake_file_minimal),
+                'os_path_exists_scenario_name': 'exist',
                 'open_scenario_name': 'okay',
                 },
-            'user': {
+            'user_xdg': {
                 'file_double_params': dict(
-                    path=user_config_file_path,
+                    path=user_xdg_config_file_path,
                     fake_file=fake_file_minimal),
+                'os_path_exists_scenario_name': 'exist',
+                'open_scenario_name': 'okay',
+                },
+            'user_home': {
+                'file_double_params': dict(
+                    path=user_home_config_file_path,
+                    fake_file=fake_file_minimal),
+                'os_path_exists_scenario_name': 'exist',
                 'open_scenario_name': 'okay',
                 },
             }
@@ -178,13 +195,16 @@ def make_config_file_scenarios():
                 'configs_by_name': {
                     'runtime': None,
                     },
+                'expected_config_names': ['global', 'user_xdg'],
                 },
             'not-exist': {
                 'configs_by_name': {
                     'runtime': {
+                        'os_path_exists_scenario_name': 'not_exist',
                         'open_scenario_name': 'nonexist',
                         },
                     },
+                'expected_config_names': [],
                 },
             'exist-read-denied': {
                 'configs_by_name': {
@@ -192,6 +212,7 @@ def make_config_file_scenarios():
                         'open_scenario_name': 'read_denied',
                         },
                     },
+                'expected_config_names': [],
                 },
             'exist-empty': {
                 'configs_by_name': {
@@ -201,6 +222,7 @@ def make_config_file_scenarios():
                             fake_file=fake_file_empty),
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-invalid': {
                 'configs_by_name': {
@@ -210,8 +232,11 @@ def make_config_file_scenarios():
                             fake_file=fake_file_bogus),
                         },
                     },
+                'expected_config_names': ['runtime'],
+                },
+            'exist-minimal': {
+                'expected_config_names': ['runtime'],
                 },
-            'exist-minimal': {},
             'exist-simple': {
                 'configs_by_name': {
                     'runtime': {
@@ -221,6 +246,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-simple-host-three': {
                 'configs_by_name': {
@@ -231,6 +257,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-method-local': {
                 'configs_by_name': {
@@ -241,6 +268,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-missing-fqdn': {
                 'configs_by_name': {
@@ -251,6 +279,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-missing-incoming': {
                 'configs_by_name': {
@@ -261,6 +290,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-default-not-unsigned': {
                 'configs_by_name': {
@@ -271,6 +301,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-default-distribution-only': {
                 'configs_by_name': {
@@ -281,6 +312,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-distribution-none': {
                 'configs_by_name': {
@@ -291,6 +323,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-distribution-one': {
                 'configs_by_name': {
@@ -301,6 +334,7 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'exist-distribution-three': {
                 'configs_by_name': {
@@ -311,14 +345,17 @@ def make_config_file_scenarios():
                         'test_section': "foo",
                         },
                     },
+                'expected_config_names': ['runtime'],
                 },
             'global-config-not-exist': {
                 'configs_by_name': {
                     'global': {
+                        'os_path_exists_scenario_name': 'not_exist',
                         'open_scenario_name': 'nonexist',
                         },
                     'runtime': None,
                     },
+                'expected_config_names': ['user_xdg'],
                 },
             'global-config-read-denied': {
                 'configs_by_name': {
@@ -327,25 +364,92 @@ def make_config_file_scenarios():
                         },
                     'runtime': None,
                     },
+                'expected_config_names': ['user_xdg'],
                 },
-            'user-config-not-exist': {
+            'user-xdg-config-not-exist': {
                 'configs_by_name': {
-                    'user': {
+                    'user_xdg': {
+                        'os_path_exists_scenario_name': 'not_exist',
+                        'open_scenario_name': 'nonexist',
+                        },
+                    'user_home': {
+                        'open_scenario_name': 'okay',
+                        },
+                    'runtime': None,
+                    },
+                'expected_config_names': ['global', 'user_home'],
+                },
+            'user-xdg-config-read-denied': {
+                'configs_by_name': {
+                    'user_xdg': {
+                        'open_scenario_name': 'read_denied',
+                        },
+                    'user_home': {
+                        'open_scenario_name': 'okay',
+                        },
+                    'runtime': None,
+                    },
+                'expected_config_names': ['global', 'user_home'],
+                },
+            'user-xdg-config-not-exist-user-home-config-not-exist': {
+                'configs_by_name': {
+                    'user_xdg': {
+                        'os_path_exists_scenario_name': 'not_exist',
+                        'open_scenario_name': 'nonexist',
+                        },
+                    'user_home': {
+                        'os_path_exists_scenario_name': 'not_exist',
                         'open_scenario_name': 'nonexist',
                         },
                     'runtime': None,
                     },
+                'expected_config_names': ['global'],
+                },
+            'user-xdg-config-not-exist-user-home-config-read-denied': {
+                'configs_by_name': {
+                    'user_xdg': {
+                        'os_path_exists_scenario_name': 'not_exist',
+                        'open_scenario_name': 'nonexist',
+                        },
+                    'user_home': {
+                        'open_scenario_name': 'read_denied',
+                        },
+                    'runtime': None,
+                    },
+                'expected_config_names': ['global'],
                 },
             'all-not-exist': {
                 'configs_by_name': {
                     'global': {
+                        'os_path_exists_scenario_name': 'not_exist',
                         'open_scenario_name': 'nonexist',
                         },
-                    'user': {
+                    'user_xdg': {
+                        'os_path_exists_scenario_name': 'not_exist',
+                        'open_scenario_name': 'nonexist',
+                        },
+                    'user_home': {
+                        'os_path_exists_scenario_name': 'not_exist',
                         'open_scenario_name': 'nonexist',
                         },
                     'runtime': None,
                     },
+                'expected_config_names': [],
+                },
+            'all-read-denied': {
+                'configs_by_name': {
+                    'global': {
+                        'open_scenario_name': 'read_denied',
+                        },
+                    'user_xdg': {
+                        'open_scenario_name': 'read_denied',
+                        },
+                    'user_home': {
+                        'open_scenario_name': 'read_denied',
+                        },
+                    'runtime': None,
+                    },
+                'expected_config_names': [],
                 },
             }
 
@@ -362,6 +466,8 @@ def make_config_file_scenarios():
                 params = default_params.copy()
                 params.update(scenario['configs_by_name'][config_name])
             params['file_double'] = FileDouble(**params['file_double_params'])
+            params['file_double'].set_os_path_exists_scenario(
+                    params['os_path_exists_scenario_name'])
             params['file_double'].set_open_scenario(
                     params['open_scenario_name'])
             params['config'] = make_config_from_stream(
@@ -383,7 +489,7 @@ def get_file_doubles_from_config_file_scenarios(scenarios):
         configs_by_name = scenario['configs_by_name']
         doubles.update(
                 configs_by_name[config_name]['file_double']
-                for config_name in ['global', 'user', 'runtime']
+                for config_name in ['global', 'user_home', 'runtime']
                 if configs_by_name[config_name] is not None)
 
     return doubles
@@ -398,6 +504,7 @@ def setup_config_file_fixtures(testcase):
     setup_file_double_behaviour(
             testcase,
             get_file_doubles_from_config_file_scenarios(scenarios.values()))
+    patch_os_path_exists(testcase)
 
 
 def set_config(testcase, name):
@@ -529,6 +636,141 @@ def patch_runtime_config_options(testcase):
                         section_name, option_name, option_value)
 
 
+def set_config_file_scenario(testcase, name):
+    """ Set the configuration file scenario for `testcase`. """
+    testcase.config_file_scenario = testcase.config_file_scenarios[name]
+    testcase.configs_by_name = testcase.config_file_scenario['configs_by_name']
+    for config_params in testcase.configs_by_name.values():
+        if config_params is not None:
+            config_params['file_double'].register_for_testcase(testcase)
+
+
+def get_path_for_runtime_config_file(testcase):
+    """ Get the path to specify for runtime config file. """
+    path = None
+    runtime_config_params = testcase.configs_by_name['runtime']
+    if runtime_config_params is not None:
+        runtime_config_file_double = runtime_config_params['file_double']
+        path = runtime_config_file_double.path
+    return path
+
+
+class active_config_files_TestCase(
+        testscenarios.WithScenarios,
+        testtools.TestCase):
+    """ Test cases for `active_config_files` function. """
+
+    scenarios = [
+            ('default', {
+                'config_scenario_name': 'default',
+                }),
+            ('exist-minimal', {
+                'config_scenario_name': 'exist-minimal',
+                }),
+            ('not-exist', {
+                'config_scenario_name': 'not-exist',
+                }),
+            ('exist-read-denied', {
+                'config_scenario_name': 'exist-read-denied',
+                }),
+            ('global-config-not-exist', {
+                'config_scenario_name': 'global-config-not-exist',
+                }),
+            ('user-xdg-config-not-exist', {
+                'config_scenario_name': 'user-xdg-config-not-exist',
+                }),
+            ('user-xdg-config-read-denied', {
+                'config_scenario_name': 'user-xdg-config-read-denied',
+                }),
+            ('user-xdg-config-not-exist-user-home-config-not-exist', {
+                'config_scenario_name':
+                    'user-xdg-config-not-exist-user-home-config-not-exist',
+                }),
+            ('user-xdg-config-not-exist-user-home-config-read-denied', {
+                'config_scenario_name':
+                    'user-xdg-config-not-exist-user-home-config-read-denied',
+                }),
+            ('all-not-exist', {
+                'config_scenario_name': 'all-not-exist',
+                }),
+            ('all-read-denied', {
+                'config_scenario_name': 'all-read-denied',
+                }),
+            ]
+
+    def setUp(self):
+        """ Set up test fixtures. """
+        super().setUp()
+        patch_system_interfaces(self)
+        setup_config_file_fixtures(self)
+        setup_file_double_behaviour(self)
+
+        set_config_file_scenario(self, self.config_scenario_name)
+        self.expected_config_names = (
+                self.config_file_scenario['expected_config_names'])
+        self.set_test_args()
+
+    def set_test_args(self):
+        """ Set the arguments for the test call to the function. """
+        runtime_config_file_path = get_path_for_runtime_config_file(self)
+        self.test_args = dict(
+                config_path=runtime_config_file_path,
+                debug=False,
+                )
+
+    def test_emits_no_skip_message_when_debug_false(self):
+        """ Should not emit a “skipping” message when `debug` is false. """
+        self.test_args['debug'] = False
+        try:
+            all(dput.configfile.active_config_files(**self.test_args))
+        except dput.configfile.ConfigurationError:
+            pass
+        unwanted_output = "skipping"
+        self.assertNotIn(unwanted_output, sys.stderr.getvalue())
+
+    def test_yields_expected_config_files(self):
+        """ Should yield a sequence of the expected configuration files. """
+        if not self.expected_config_names:
+            self.skipTest("Normal return not expected in this scenario")
+        expected_files_list = [
+                self.configs_by_name[config_name]['file_double'].fake_file
+                for config_name in self.expected_config_names]
+        result_list = list(
+                dput.configfile.active_config_files(**self.test_args))
+        self.assertEqual(expected_files_list, result_list)
+
+    def test_raise_error_if_all_config_files_fail(self):
+        """
+        Should raise `ConfigurationError` if unable to open any config files.
+        """
+        if self.expected_config_names:
+            self.skipTest("No error expected for this scenario")
+        expected_message_regex = (
+                "^Could not open any configfile, tried .*")
+        with testtools.ExpectedException(
+                dput.configfile.ConfigurationError,
+                value_re=expected_message_regex):
+            all(dput.configfile.active_config_files(**self.test_args))
+
+
+def patch_active_config_files(testcase):
+    """ Patch the `active_config_files` function for `testcase`. """
+    if not hasattr(testcase, 'fake_config_files'):
+        testcase.fake_config_files = [
+                FileDouble(path)
+                for path in testcase.getUniqueString() for __ in range(3)]
+
+    def fake_active_config_files(*args, **kwargs):
+        for config_file in testcase.fake_config_files:
+            yield config_file
+
+    func_patcher = unittest.mock.patch.object(
+            dput.configfile, "active_config_files", autospec=True,
+            side_effect=fake_active_config_files)
+    func_patcher.start()
+    testcase.addCleanup(func_patcher.stop)
+
+
 class read_configs_TestCase(testtools.TestCase):
     """ Test cases for `read_config` function. """
 
@@ -549,31 +791,17 @@ class read_configs_TestCase(testtools.TestCase):
         patcher_class_configparser.start()
         self.addCleanup(patcher_class_configparser.stop)
 
-        self.set_config_file_scenario('exist-minimal')
+        set_config_file_scenario(self, 'exist-minimal')
         self.set_test_args()
 
-    def set_config_file_scenario(self, name):
-        """ Set the configuration file scenario for this test case. """
-        self.config_file_scenario = self.config_file_scenarios[name]
-        self.configs_by_name = self.config_file_scenario['configs_by_name']
-        for config_params in self.configs_by_name.values():
-            if config_params is not None:
-                config_params['file_double'].register_for_testcase(self)
-
-    def get_path_for_runtime_config_file(self):
-        """ Get the path to specify for runtime config file. """
-        path = ""
-        runtime_config_params = self.configs_by_name['runtime']
-        if runtime_config_params is not None:
-            runtime_config_file_double = runtime_config_params['file_double']
-            path = runtime_config_file_double.path
-        return path
-
     def set_test_args(self):
         """ Set the arguments for the test call to the function. """
-        runtime_config_file_path = self.get_path_for_runtime_config_file()
+        runtime_config_file_path = get_path_for_runtime_config_file(self)
+        config_files = [
+                config_params['file_double'].fake_file
+                for config_params in self.configs_by_name.values()]
         self.test_args = dict(
-                extra_config=runtime_config_file_path,
+                config_files=config_files,
                 debug=False,
                 )
 
@@ -612,106 +840,25 @@ class read_configs_TestCase(testtools.TestCase):
         result = dput.dput.read_configs(**self.test_args)
         self.assertTrue(option_names.issubset(set(result.defaults().keys())))
 
-    def test_opens_default_config_files(self):
-        """ Should open the default config files. """
-        self.set_config_file_scenario('default')
-        self.set_test_args()
-        dput.dput.read_configs(**self.test_args)
-        expected_calls = [
-                unittest.mock.call(
-                    self.configs_by_name[config_name]['file_double'].path)
-                for config_name in ['global', 'user']]
-        builtins.open.assert_has_calls(expected_calls)
-
-    def test_opens_specified_config_file(self):
-        """ Should open the specified config file. """
-        dput.dput.read_configs(**self.test_args)
-        builtins.open.assert_called_with(
-                self.configs_by_name['runtime']['file_double'].path)
-
-    def test_emits_debug_message_on_opening_config_file(self):
-        """ Should emit a debug message when opening the config file. """
+    def test_emits_debug_message_on_parsing_config_file(self):
+        """ Should emit a debug message when parsing each config file. """
         self.test_args['debug'] = True
         config_file_double = self.configs_by_name['runtime']['file_double']
         dput.dput.read_configs(**self.test_args)
         expected_output = textwrap.dedent("""\
-                D: Parsing Configuration File {path}
+                D: Parsing configuration file ‘{path}’
                 """).format(path=config_file_double.path)
         self.assertIn(expected_output, sys.stdout.getvalue())
 
-    def test_emits_no_skip_message_when_debug_false(self):
-        """ Should not emit a “skipping” message when `debug` is false. """
-        self.set_config_file_scenario('global-config-not-exist')
-        self.set_test_args()
-        config_file_double = self.configs_by_name['global']['file_double']
-        self.test_args['debug'] = False
-        dput.dput.read_configs(**self.test_args)
-        expected_calls = [
-                unittest.mock.call(config_file_double.path)]
-        unwanted_output = "skipping"
-        builtins.open.assert_has_calls(expected_calls)
-        self.assertNotIn(unwanted_output, sys.stderr.getvalue())
-
-    def test_skips_file_if_not_exist(self):
-        """ Should skip a config file if it doesn't exist. """
-        self.set_config_file_scenario('global-config-not-exist')
-        self.set_test_args()
-        config_file_double = self.configs_by_name['global']['file_double']
-        self.test_args['debug'] = True
-        dput.dput.read_configs(**self.test_args)
-        expected_calls = [
-                unittest.mock.call(config_file_double.path)]
-        expected_output = textwrap.dedent("""\
-                No such file ...: {path}, skipping
-                """).format(path=config_file_double.path)
-        builtins.open.assert_has_calls(expected_calls)
-        self.assertThat(
-                sys.stderr.getvalue(),
-                testtools.matchers.DocTestMatches(
-                    expected_output, flags=doctest.ELLIPSIS))
-
-    def test_skips_file_if_permission_denied(self):
-        """ Should skip a config file if read permission is denied.. """
-        self.set_config_file_scenario('global-config-read-denied')
-        self.set_test_args()
-        config_file_double = self.configs_by_name['global']['file_double']
-        self.test_args['debug'] = True
-        dput.dput.read_configs(**self.test_args)
-        expected_calls = [
-                unittest.mock.call(config_file_double.path)]
-        expected_output = textwrap.dedent("""\
-                Read denied on ...: {path}, skipping
-                """).format(path=config_file_double.path)
-        builtins.open.assert_has_calls(expected_calls)
-        self.assertThat(
-                sys.stderr.getvalue(),
-                testtools.matchers.DocTestMatches(
-                    expected_output, flags=doctest.ELLIPSIS))
-
-    def test_calls_sys_exit_if_no_config_files(self):
-        """ Should call `sys.exit` if unable to open any config files. """
-        self.set_config_file_scenario('all-not-exist')
-        self.set_test_args()
-        with testtools.ExpectedException(FakeSystemExit):
-            dput.dput.read_configs(**self.test_args)
-        expected_output = textwrap.dedent("""\
-                Error: Could not open any configfile, tried ...
-                """)
-        self.assertThat(
-                sys.stderr.getvalue(),
-                testtools.matchers.DocTestMatches(
-                    expected_output, flags=doctest.ELLIPSIS))
-        sys.exit.assert_called_with(EXIT_STATUS_FAILURE)
-
     def test_calls_sys_exit_if_config_parsing_error(self):
         """ Should call `sys.exit` if a parsing error occurs. """
-        self.set_config_file_scenario('exist-invalid')
+        set_config_file_scenario(self, 'exist-invalid')
         self.set_test_args()
         self.test_args['debug'] = True
         with testtools.ExpectedException(FakeSystemExit):
             dput.dput.read_configs(**self.test_args)
         expected_output = textwrap.dedent("""\
-                Error parsing config file:
+                Error parsing configuration file:
                 ...
                 """)
         self.assertThat(
@@ -722,7 +869,7 @@ class read_configs_TestCase(testtools.TestCase):
 
     def test_sets_fqdn_option_if_local_method(self):
         """ Should set “fqdn” option for “local” method. """
-        self.set_config_file_scenario('exist-method-local')
+        set_config_file_scenario(self, 'exist-method-local')
         self.set_test_args()
         result = dput.dput.read_configs(**self.test_args)
         runtime_config_params = self.configs_by_name['runtime']
@@ -731,7 +878,7 @@ class read_configs_TestCase(testtools.TestCase):
 
     def test_exits_with_error_if_missing_fqdn(self):
         """ Should exit with error if config is missing 'fqdn'. """
-        self.set_config_file_scenario('exist-missing-fqdn')
+        set_config_file_scenario(self, 'exist-missing-fqdn')
         self.set_test_args()
         with testtools.ExpectedException(FakeSystemExit):
             dput.dput.read_configs(**self.test_args)
@@ -743,7 +890,7 @@ class read_configs_TestCase(testtools.TestCase):
 
     def test_exits_with_error_if_missing_incoming(self):
         """ Should exit with error if config is missing 'incoming'. """
-        self.set_config_file_scenario('exist-missing-incoming')
+        set_config_file_scenario(self, 'exist-missing-incoming')
         self.set_test_args()
         with testtools.ExpectedException(FakeSystemExit):
             dput.dput.read_configs(**self.test_args)
@@ -770,7 +917,7 @@ class print_config_TestCase(testtools.TestCase):
         mock_config.write.assert_called_with(sys.stdout)
 
 
-# Copyright © 2015–2021 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2022 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
