@@ -40,6 +40,7 @@ from .test_changesfile import (
         setup_changes_file_fixtures,
         )
 from .test_configfile import (
+        patch_active_config_files,
         patch_runtime_config_options,
         set_config,
         )
@@ -349,6 +350,7 @@ class main_TestCase(
         self.options_defaults = dict(
                 dput.dput.parse_commandline.options_defaults)
         self.patch_parse_commandline()
+        patch_active_config_files(self)
         patch_read_configs(self)
         patch_print_config(self)
         patch_import_upload_functions(self)
@@ -549,20 +551,66 @@ class main_ConfigFileTestCase(
 
     scenarios = [
             ('default', {
-                'expected_args': ("", unittest.mock.ANY),
+                'expected_config_path': "",
                 }),
             ('config-from-command-line', {
                 'fake_options_mapping': {
                     'config_file_path': "lorem.conf",
                     },
-                'expected_args': ("lorem.conf", unittest.mock.ANY),
+                'expected_config_path': "lorem.conf",
                 }),
             ]
 
+    def test_calls_active_config_files_with_expected_args(self):
+        """ Should call `active_config_files` with expected arguments. """
+        expected_args = (self.expected_config_path, unittest.mock.ANY)
+        self.function_to_test(**self.test_args)
+        dput.configfile.active_config_files.assert_called_with(*expected_args)
+
     def test_calls_read_configs_with_expected_args(self):
         """ Should call `read_configs` with expected arguments. """
+        expected_config_files = self.fake_config_files
+        expected_args = (expected_config_files, unittest.mock.ANY)
         self.function_to_test(**self.test_args)
-        dput.dput.read_configs.assert_called_with(*self.expected_args)
+        dput.dput.read_configs.assert_called_with(*expected_args)
+
+    def test_calls_sys_exit_if_configuration_error_when_opening_files(self):
+        """
+        Should call `sys.exit` if `ConfigurationError` from opening files.
+        """
+        set_config(self, 'all-not-exist')
+        self.set_test_args()
+        fake_error_text = "Could not open any configfile, tried ‘foo’, ‘bar’"
+        fake_error = dput.configfile.ConfigurationError(fake_error_text)
+        dput.configfile.active_config_files.side_effect = fake_error
+        with testtools.ExpectedException(FakeSystemExit):
+            self.function_to_test(**self.test_args)
+        expected_output = textwrap.dedent("""\
+                ...{error_text}...
+                """.format(error_text=fake_error_text))
+        self.assertThat(
+                sys.stderr.getvalue(),
+                testtools.matchers.DocTestMatches(
+                    expected_output, flags=doctest.ELLIPSIS))
+        sys.exit.assert_called_with(EXIT_STATUS_FAILURE)
+
+    def test_calls_sys_exit_if_configuration_error_when_reading_files(self):
+        """
+        Should call `sys.exit` if `ConfigurationError` from reading files.
+        """
+        fake_error_text = "Hidden Illuminati messages in config file"
+        fake_error = dput.configfile.ConfigurationError(fake_error_text)
+        dput.dput.read_configs.side_effect = fake_error
+        with testtools.ExpectedException(FakeSystemExit):
+            self.function_to_test(**self.test_args)
+        expected_output = textwrap.dedent("""\
+                ...{error_text}...
+                """.format(error_text=fake_error_text))
+        self.assertThat(
+                sys.stderr.getvalue(),
+                testtools.matchers.DocTestMatches(
+                    expected_output, flags=doctest.ELLIPSIS))
+        sys.exit.assert_called_with(EXIT_STATUS_FAILURE)
 
 
 class main_PrintAndEndTestCase(main_TestCase):
@@ -1384,7 +1432,7 @@ class main_DinstallTestCase(
         dput.dput.dinstall_caller.assert_called_with(*expected_args)
 
 
-# Copyright © 2015–2021 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2022 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
